package com.monead.androjena.demo.arqoid;


import java.util.List;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.Syntax;

/**
 * Demonstrate some SPARQL operations using ARQoid
 * 
 * Built in support of a series of blog entries in the use of
 * ARQoid.
 * 
 * @version 1.0 - 20111129
 * @author David Read
 */
public class SparqlExamples {
    /**
     * A trivial use of ARQoid to output CSV report of results from a SPARQL
     * endpoint.
     */
    public String queryRemoteSparqlEndpoint() {
        /**
         * Use the SPARQL engine and report the results
         * 
         * @return The number of resulting rows
         */

        // Set the query
        String queryString = "SELECT ?dataType ?data " 
            + "WHERE {"
            + "<http://nasa.dataincubator.org/launch/1961-012> "
            + "?dataType ?data."
            + "}";

        // Set the SPARQL endpoint URI
        String sparqlEndpointUri = "http://api.talis.com/stores/space/services/sparql";

        // Create a Query instance
        Query query = QueryFactory.create(queryString, Syntax.syntaxARQ);

        // Limit the number of results returned
        // Setting the limit is optional - default is unlimited
        query.setLimit(10);

        // Set the starting record for results returned
        // Setting the limit is optional - default is 1 (and it is 1-based)
        query.setOffset(1);

        // Query uses an external SPARQL endpoint for processing
        // This is the syntax for that type of query
        QueryExecution qe = QueryExecutionFactory.sparqlService(sparqlEndpointUri, query);

        // Execute the query and obtain results
        ResultSet resultSet = qe.execSelect();

        // Setup a place to house results for output
        StringBuffer results = new StringBuffer();

        // Get the column names (the aliases supplied in the SELECT clause)
        List<String> columnNames = resultSet.getResultVars();

        // Iterate through all resulting rows
        while (resultSet.hasNext()) {
            // Get the next result row
            QuerySolution solution = resultSet.next();

            // Iterate through the columns
            for (String var : columnNames) {
                // Add the column label to the StringBuffer
                results.append(var + ": ");

                // Add the returned row/column data to the StringBuffer
                
                // Data value will be null if optional and not present
                if (solution.get(var) == null) {
                    results.append("{null}");
                // Test whether the returned value is a literal value
                } else if (solution.get(var).isLiteral()) {
                    results.append(solution.getLiteral(var).toString());
                // Otherwise the returned value is a URI
                } else {
                    results.append(solution.getResource(var).getURI());
                }
                results.append('\n');
            }
            results.append("-----------------\n");
        }

        // Important - free up resources used running the query
        qe.close();
        
        // Return the results as a String
        return results.toString();
    }
}
